import React from 'react';
import { createPortal } from 'react-dom';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var INITIAL_VALUE = {
  node: null,
  animatedIndex: null,
  screens: []
};
var Context = React.createContext(INITIAL_VALUE);

var Provider =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Provider, _React$Component);

  function Provider() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;

    _defineProperty(_assertThisInitialized(_this), "contentNode", React.createRef());

    _defineProperty(_assertThisInitialized(_this), "state", _objectSpread({}, INITIAL_VALUE, {
      startAnimation: function startAnimation(animatedIndex) {
        _this.setState(function (state) {
          return state.animatedIndex ? undefined : {
            animatedIndex: animatedIndex
          };
        });
      },
      finishAnimation: function finishAnimation() {
        return _this.setState({
          animatedIndex: null
        });
      },
      addScreen: function addScreen(index) {
        _this.setState(function (state) {
          return {
            screens: [].concat(state.screens.filter(function (s) {
              return s !== index;
            }), [index])
          };
        });
      },
      removeScreen: function removeScreen(index) {
        _this.setState(function (state) {
          return {
            screens: state.screens.filter(function (s) {
              return s !== index;
            })
          };
        });
      }
    }));

    return _this;
  }

  var _proto = Provider.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.setState({
      node: this.contentNode.current
    });
  };

  _proto.render = function render() {
    var children = this.props.children;
    var _this$state = this.state,
        animatedIndex = _this$state.animatedIndex,
        screens = _this$state.screens;
    var shift = 100 * screens.length;
    return React.createElement(Context.Provider, {
      value: this.state
    }, React.createElement("div", {
      style: styles.container
    }, React.createElement("div", {
      ref: this.contentNode,
      style: _objectSpread({}, styles.content, {
        transform: "translateX(-" + shift + "vw)"
      })
    }, animatedIndex && React.createElement("div", {
      style: _objectSpread({}, styles.overlay, {
        right: "-" + shift + "vw"
      })
    }), React.createElement("div", {
      style: styles.view
    }, children))));
  };

  return Provider;
}(React.Component);

var styles = {
  container: {
    height: '100%',
    overflow: 'hidden'
  },
  content: {
    height: '100%',
    display: 'flex'
  },
  view: {
    boxSizing: 'border-box',
    minHeight: '100%',
    width: '100vw',
    flexShrink: 0,
    overflow: 'auto',
    position: 'relative'
  },
  overlay: {
    position: 'absolute',
    zIndex: 1,
    top: 0,
    bottom: 0,
    right: 0,
    width: '100vw',
    margin: 'auto'
  }
};

var ANIMATION_DURATION = 500;
var DEFAULT_STATE = {
  openStarted: false,
  opened: false,
  beforeClose: false,
  closeStarted: false
};
var viewIndex = 1;

var View =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(View, _React$Component);

  function View() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;

    _defineProperty(_assertThisInitialized(_this), "index", ++viewIndex);

    _defineProperty(_assertThisInitialized(_this), "currentAnimation", null);

    _defineProperty(_assertThisInitialized(_this), "state", _objectSpread({}, DEFAULT_STATE, {
      active: false
    }));

    _defineProperty(_assertThisInitialized(_this), "clearAnimationTimeout", function () {
      if (_this.currentAnimation) clearTimeout(_this.currentAnimation);
    });

    _defineProperty(_assertThisInitialized(_this), "setAnimationTimeout", function (fn, timeout) {
      if (timeout === void 0) {
        timeout = 20;
      }

      clearTimeout(_this.currentAnimation);
      _this.currentAnimation = setTimeout(fn, timeout);
    });

    _defineProperty(_assertThisInitialized(_this), "startOpen", function () {
      return _this.setAnimationTimeout(_this.open);
    });

    _defineProperty(_assertThisInitialized(_this), "open", function () {
      _this.setState(_objectSpread({}, DEFAULT_STATE, {
        openStarted: true
      }), function () {
        _this.setAnimationTimeout(_this.finishOpen, ANIMATION_DURATION);
      });
    });

    _defineProperty(_assertThisInitialized(_this), "finishOpen", function () {
      _this.clearAnimationTimeout();

      _this.setState(_objectSpread({}, DEFAULT_STATE, {
        opened: true
      }), function () {
        _this.context.finishAnimation();

        _this.context.addScreen(_this.index);
      });
    });

    _defineProperty(_assertThisInitialized(_this), "startClose", function () {
      _this.setState(_objectSpread({}, DEFAULT_STATE, {
        beforeClose: true
      }), function () {
        _this.context.removeScreen(_this.index);

        _this.setAnimationTimeout(_this.close);
      });
    });

    _defineProperty(_assertThisInitialized(_this), "close", function () {
      _this.setState(_objectSpread({}, DEFAULT_STATE, {
        closeStarted: true
      }), function () {
        _this.setAnimationTimeout(_this.finishClose, ANIMATION_DURATION);
      });
    });

    _defineProperty(_assertThisInitialized(_this), "finishClose", function () {
      _this.setAnimationTimeout(function () {
        _this.context.finishAnimation();

        _this.setState(DEFAULT_STATE);
      });
    });

    return _this;
  }

  var _proto = View.prototype;

  _proto.componentDidUpdate = function componentDidUpdate() {
    var _this2 = this;

    var active = this.props.active;

    if (active !== this.state.active) {
      if (this.context.animatedIndex === this.index) {
        this.setState({
          active: active
        }, function () {
          _this2.state.active ? _this2.startOpen() : _this2.state.openStarted ? _this2.close() : _this2.startClose();
        });
        return;
      }

      if (!this.context.animatedIndex) this.context.startAnimation(this.index);
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    if (this.state.opened) this.context.removeScreen(this.index);
    this.clearAnimationTimeout();
  };

  _proto.render = function render() {
    var _this$state = this.state,
        active = _this$state.active,
        openStarted = _this$state.openStarted,
        opened = _this$state.opened,
        beforeClose = _this$state.beforeClose,
        closeStarted = _this$state.closeStarted;
    var node = this.context.node;
    var visible = node && (active || openStarted || beforeClose || closeStarted);
    if (!visible) return null;

    var style = _objectSpread({}, styles$1.container, {
      transform: openStarted || beforeClose ? 'translateX(-100vw)' : 'none'
    });

    if (!opened) {
      style.zIndex = 1;
      style.transition = ANIMATION_DURATION + "ms";
    }

    return createPortal(React.createElement('div', {
      style: style
    }, this.props.children()), node);
  };

  return View;
}(React.Component);

_defineProperty(View, "contextType", Context);

var styles$1 = {
  container: {
    boxSizing: 'border-box',
    willChange: 'transform',
    minHeight: '100%',
    width: '100vw',
    flexShrink: 0,
    overflow: 'auto',
    background: '#fff'
  }
};

export { Provider, View };
